#!C:\Users\dev-admin\perseverance-python-buildout\croot\pythonanywhere_1699565098275\_h_env\python.exe
"""Autoconfigure a Django project from on a github URL.

- downloads the repo
- creates a virtualenv and installs django 1.x (or detects a requirements.txt if available)
- creates webapp via api
- creates django wsgi configuration file
- adds static files config

Usage:
  pa_autoconfigure_django.py <git-repo-url> [--branch=<branch> --domain=<domain> --python=<python-version>] [--nuke]

Options:
  --branch=<branch>         Branch name in case of multiple branches   [default: None]
  --domain=<domain>         Domain name, eg www.mydomain.com   [default: your-username.pythonanywhere.com]
  --python=<python-version> Python version, eg "3.8"    [default: 3.7]
  --nuke                    *Irrevocably* delete any existing web app config on this domain. Irrevocably.
"""

from docopt import docopt

from pythonanywhere.django_project import DjangoProject
from pythonanywhere.snakesay import snakesay
from pythonanywhere.utils import ensure_domain


def main(repo_url, branch, domain, python_version, nuke):
    domain = ensure_domain(domain)
    project = DjangoProject(domain, python_version)
    project.sanity_checks(nuke=nuke)
    project.download_repo(repo_url, nuke=nuke),
    project.ensure_branch(branch),
    project.create_virtualenv(nuke=nuke)
    project.create_webapp(nuke=nuke)
    project.add_static_file_mappings()
    project.find_django_files()
    project.update_wsgi_file()
    project.update_settings_file()
    project.run_collectstatic()
    project.run_migrate()
    project.webapp.reload()
    print(snakesay(f'All done!  Your site is now live at https://{domain}'))
    print()
    project.start_bash()


if __name__ == '__main__':
    arguments = docopt(__doc__)
    main(
        arguments['<git-repo-url>'],
        arguments['--branch'],
        arguments['--domain'],
        arguments['--python'],
        nuke=arguments.get('--nuke')
    )



import getpass
import os


def ensure_domain(domain):
    if domain == "your-username.pythonanywhere.com":
        username = getpass.getuser().lower()
        pa_domain = os.environ.get("PYTHONANYWHERE_DOMAIN", "pythonanywhere.com")
        return f"{username}.{pa_domain}"
    else:
        return domain



import json
import re
import sys

from collections import namedtuple
from pprint import pprint

import typer

from pythonanywhere.files import PAPath
from pythonanywhere.scripts_commons import get_logger

app = typer.Typer()


def setup(path: str, quiet: bool) -> PAPath:
    logger = get_logger(set_info=True)
    if quiet:
        logger.disabled = True
    return PAPath(path)


@app.command()
def get(
    path: str = typer.Argument(..., help="Path to PythonAnywhere file or directory."),
    only_files: bool = typer.Option(False, "-f", "--files", help="List only files."),
    only_dirs: bool = typer.Option(False, "-d", "--dirs", help="List only directories."),
    sort_by_type: bool = typer.Option(False, "-t", "--type", help="Sort by type."),
    sort_reverse: bool = typer.Option(False, "-r", "--reverse", help="Sort in reverse order."),
    raw: bool = typer.Option(
        False, "-a", "--raw", help="Print API response (has effect only for directories)."
    ),
    quiet: bool = typer.Option(False, "-q", "--quiet", help="Disable additional logging."),
):
    """
    Get contents of PATH.

    If PATH points to a directory, show list of it's contents.
    If PATH points to a file, print it's contents.
    """
    pa_path = setup(path, quiet)
    contents = pa_path.contents

    if contents is None:
        sys.exit(1)

    if raw or isinstance(contents, str):
        {dict: lambda x: print(json.dumps(x)), str: print}[type(contents)](contents)
        sys.exit()

    NameToType = namedtuple("NameToType", ["name", "type"])
    item = "file" if only_files else "directory" if only_dirs else "every"
    data = [NameToType(k, v["type"]) for k, v in contents.items()]

    if sort_reverse or sort_by_type:
        data.sort(key=lambda x: x.type if sort_by_type else x.name, reverse=sort_reverse)

    typer.echo(f"{pa_path.path}:")
    for name, type_ in data:
        if item == "every":
            typer.echo(f"{type_[0].upper()}  {name}")
        elif type_ == item:
            typer.echo(f"   {name}")


def _format_tree(data, current):
    last_child = "└── "
    next_child = "├── "
    connector  = "│   "
    filler     = "    "

    formatted = []
    level_tracker = set()

    for entry in reversed(data):
        entry = re.sub(r"/$", "\0", entry.replace(current, ""))
        chunks = [cc for cc in entry.split("/") if cc]
        item = chunks[-1].replace("\0", "/")
        level = len(chunks) - 1
        level_tracker = set([lvl for lvl in level_tracker if lvl <= level])
        indents = [connector if lvl in level_tracker else filler for lvl in range(level)]
        indents.append(last_child if level not in level_tracker else next_child)
        level_tracker.add(level)
        formatted.append("".join(indents) + item)

    return "\n".join(reversed(formatted))


@app.command()
def tree(
    path: str = typer.Argument(..., help="Path to PythonAnywhere directory."),
    quiet: bool = typer.Option(False, "-q", "--quiet", help="Disable additional logging.")
):
    """Show preview of directory contents at PATH in tree-like format (2 levels deep)."""
    pa_path = setup(path, quiet)
    tree = pa_path.tree

    if tree is not None:
        typer.echo(f"{pa_path.path}:")
        typer.echo(".")
        typer.echo(_format_tree(tree, pa_path.path))
    else:
        sys.exit(1)


@app.command()
def upload(
    path: str = typer.Argument(..., help=("Full path of FILE where CONTENTS should be uploaded to.")),
    file: typer.FileBinaryRead = typer.Option(
        ...,
        "-c",
        "--contents",
        help="Path to exisitng file or stdin stream that should be uploaded to PATH."
    ),
    quiet: bool = typer.Option(False, "-q", "--quiet", help="Disable additional logging.")
):
    """
    Upload CONTENTS to file at PATH.

    If PATH points to an existing file, it will be overwritten.
    """
    pa_path = setup(path, quiet)
    success = pa_path.upload(file)
    sys.exit(0 if success else 1)


@app.command()
def delete(
    path: str = typer.Argument(..., help="Path to PythonAnywhere file or directory to be deleted."),
    quiet: bool = typer.Option(False, "-q", "--quiet", help="Disable additional logging.")
):
    """
    Delete file or directory at PATH.

    If PATH points to a user owned directory all its contents will be
    deleted recursively.
    """
    pa_path = setup(path, quiet)
    success = pa_path.delete()
    sys.exit(0 if success else 1)


@app.command()
def share(
    path: str = typer.Argument(..., help="Path to PythonAnywhere file."),
    check: bool = typer.Option(False, "-c", "--check", help="Check sharing status."),
    porcelain: bool = typer.Option(False, "-p", "--porcelain", help="Return sharing url in easy-to-parse format."),
    quiet: bool = typer.Option(False, "-q", "--quiet", help="Disable logging."),
):
    """Create a sharing link to a file at PATH or check its sharing status."""
    pa_path = setup(path, quiet or porcelain)
    link = pa_path.get_sharing_url() if check else pa_path.share()

    if not link:
        sys.exit(1)
    if porcelain:
        typer.echo(link)


@app.command()
def unshare(
    path: str = typer.Argument(..., help="Path to PythonAnywhere file."),
    quiet: bool = typer.Option(False, "-q", "--quiet", help="Disable additional logging.")
):
    """Disable sharing link for a file at PATH."""
    pa_path = setup(path, quiet)
    success = pa_path.unshare()
    sys.exit(0 if success else 1)



#!C:\Users\dev-admin\perseverance-python-buildout\croot\pythonanywhere_1699565098275\_h_env\python.exe
"""Set the HTTPS certificate and private key for a website, assuming that these have been generated by the dehydrated
script that gets them from Let's Encrypt, and that they're in the standard place.  This script should normally only
be run on PythonAnywhere.

Usage:
  pa_install_webapp_letsencrypt_ssl.py <domain> [--suppress-reload]

Options:
  <domain>              Domain name, eg www.mydomain.com
  --suppress-reload     The website will need to be reloaded in order to activate the new certificate/key combination
                        -- this happens by default, use this option to suppress it.
"""

from docopt import docopt
from os.path import expanduser
import os
import sys

from pythonanywhere.api.webapp import Webapp
from pythonanywhere.snakesay import snakesay


def main(domain_name, suppress_reload):
    homedir = expanduser("~")
    possible_paths = (
        os.path.join(homedir, 'letsencrypt', domain_name),
        os.path.join(homedir, 'letsencrypt', 'certs', domain_name),
    )
    done = False
    for path in possible_paths:
        certificate_file = os.path.join(path, 'fullchain.pem')
        private_key_file = os.path.join(path, 'privkey.pem')
        if os.path.exists(certificate_file) and os.path.exists(private_key_file):
            with open(certificate_file, "r") as f:
                certificate = f.read()
            with open(private_key_file, "r") as f:
                private_key = f.read()

            webapp = Webapp(domain_name)
            webapp.set_ssl(certificate, private_key)
            if not suppress_reload:
                webapp.reload()

            ssl_details = webapp.get_ssl_info()
            print(
                snakesay(
                    "This method of handling Let's Encrypt certs\n"
                    "**************IS DEPRECATED.**************\n\n"
                    "You can now have a Let's Encrypt certificate managed by PythonAnywhere.\n"
                    "We handle all the details of getting it, installing it,\n"
                    "and managing renewals for you. So you don't need to do\n"
                    "any of the stuff below any more.\n\n"
                    "Anyway, All is set up for now. \n"
                    "Your new certificate will expire on {expiry:%d %B %Y},\n"
                    "so shortly before then you should switch to the new system\n"
                    "(see https://help.pythonanywhere.com/pages/HTTPSSetup/)\n"
                    "".format(
                        expiry=ssl_details["not_after"]
                    )
                )
            )

            done = True
            break

    if not done:
        print(f"Could not find certificate or key files (looked in {possible_paths})")
        sys.exit(2)


if __name__ == '__main__':
    arguments = docopt(__doc__)
    main(
        arguments['<domain>'],
        suppress_reload=arguments.get('--suppress-reload')
    )



#!C:\Users\dev-admin\perseverance-python-buildout\croot\pythonanywhere_1699565098275\_h_env\python.exe
"""Deletes webapp logs.

- gets list of logs via api
- deletes logs via api

Usage:
  pa_delete_webapp_logs.py [--domain=<domain>] [--log_type=<log_type>] [--log_index=<log_index>]

Options:
  --domain=<domain>         Domain name, eg www.mydomain.com   [default: your-username.pythonanywhere.com]
  --log_type=<log_type>     Log type, could be access, error, server or all   [default: all]
  --log_index=<log_index>   Log index, 0 for current log, 1-9 for one of archive logs or all [default: all]
"""

from docopt import docopt

from pythonanywhere.api.webapp import Webapp
from pythonanywhere.snakesay import snakesay
from pythonanywhere.utils import ensure_domain


def main(domain, log_type, log_index):
    webapp = Webapp(ensure_domain(domain))
    log_types = ["access", "error", "server"]
    logs = webapp.get_log_info()
    if log_type == "all" and log_index == "all":
        for key in log_types:
            for log in logs[key]:
                webapp.delete_log(key, log)
    elif log_type == "all":
        for key in log_types:
            webapp.delete_log(key, int(log_index))
    elif log_index == "all":
        for i in logs[log_type]:
            webapp.delete_log(log_type, int(i))
    else:
        webapp.delete_log(log_type, int(log_index))
    print(snakesay('All Done!'))


if __name__ == '__main__':
    arguments = docopt(__doc__)
    main(arguments['--domain'], arguments['--log_type'], arguments['--log_index'])



#!C:\Users\dev-admin\perseverance-python-buildout\croot\pythonanywhere_1699565098275\_h_env\python.exe
"""Delete scheduled task(s) by id or nuke'em all.

Usage:
  pa_delete_scheduled_task.py id <num>...
  pa_delete_scheduled_task.py nuke [--force]

Options:
  -h, --help                  Prints this message
  -f, --force                 Turns off user confirmation before deleting tasks

Note:
  Task id <num> may be acquired with `pa_get_scheduled_tasks_list.py` script."""

from docopt import docopt

from pythonanywhere.scripts_commons import ScriptSchema, get_logger, get_task_from_id
from pythonanywhere.task import TaskList


def _delete_all(force):
    if not force:
        if input("This will irrevocably delete all your tasks, proceed? [y/N] ").lower() != "y":
            return None

    for task in TaskList().tasks:
        task.delete_schedule()


def _delete_by_id(id_numbers):
    for task_id in id_numbers:
        task = get_task_from_id(task_id, no_exit=True)
        task.delete_schedule()


def main(*, id_numbers, nuke, force):
    get_logger(set_info=True)

    if nuke:
        _delete_all(force)
    else:
        _delete_by_id(id_numbers)


if __name__ == "__main__":
    schema = ScriptSchema(
        {"id": bool, "<num>": ScriptSchema.id_multi, "nuke": bool, "--force": ScriptSchema.boolean}
    )
    arguments = schema.validate_user_input(docopt(__doc__), conversions={"num": "id_numbers"})
    arguments.pop("id")

    main(**arguments)



""" Interface speaking with PythonAnywhere API providing methods for scheduled tasks.
*Don't use* `Schedule` :class: in helper scripts, use `task.Task` instead."""

import getpass

from pythonanywhere.api.base import call_api, get_api_endpoint


class Schedule:
    """Interface for PythonAnywhere scheduled tasks API.

    Uses `pythonanywhere.api` :method: `get_api_endpoint` to create url,
    which is stored in a class variable `Schedule.base_url`, then calls
    `call_api` with appropriate arguments to execute scheduled tasks tasks
    actions. Covers 'GET' and 'POST' methods for tasks list, as well as
    'GET', 'PATCH' and 'DELETE' methods for task with id.

    Use :method: `Schedule.get_list` to get all tasks list.
    Use :method: `Schedule.create` to create new task.
    Use :method: `Schedule.get_specs` to get existing task specs.
    Use :method: `Schedule.delete` to delete existing task.
    Use :method: `Schedule.update` to update existing task."""

    base_url = get_api_endpoint().format(username=getpass.getuser(), flavor="schedule")

    def get_list(self):
        """Gets list of existing scheduled tasks.

        :returns: list of existing scheduled tasks specs"""

        return call_api(self.base_url, "GET").json()

    def create(self, params):
        """Creates new scheduled task using `params`.

        Params should be: command, enabled (True or False), interval (daily or
        hourly), hour (24h format) and minute.

        :param params: dictionary with required scheduled task specs
        :returns: dictionary with created task specs"""

        result = call_api(self.base_url, "POST", json=params)

        if result.status_code == 201:
            return result.json()

        if not result.ok:
            raise Exception(
                f"POST to set new task via API failed, got {result}: {result.text}"
            )

    def get_specs(self, task_id):
        """Get task specs by id.

        :param task_id: existing task id
        :returns: dictionary of existing task specs"""

        result = call_api(
            f"{self.base_url}{task_id}/", "GET"
        )
        if result.status_code == 200:
            return result.json()
        else:
            raise Exception(
                "Could not get task with id {task_id}. Got result {result}: {content}".format(
                    task_id=task_id, result=result, content=result.text
                )
            )

    def delete(self, task_id):
        """Deletes scheduled task by id.

        :param task_id: scheduled task to be deleted id number
        :returns: True when API response is 204"""

        result = call_api(
            f"{self.base_url}{task_id}/", "DELETE"
        )

        if result.status_code == 204:
            return True

        if not result.ok:
            raise Exception(
                f"DELETE via API on task {task_id} failed, got {result}: {result.text}"
            )

    def update(self, task_id, params):
        """Updates existing task using id and params.

        Params should at least one of: command, enabled, interval, hour,
        minute. To update hourly task don't use 'hour' param. On the other
        hand when changing task's interval from 'hourly' to 'daily' hour is
        required.

        :param task_id: existing task id
        :param params: dictionary of specs to update"""

        result = call_api(
            f"{self.base_url}{task_id}/",
            "PATCH",
            json=params,
        )
        if result.status_code == 200:
            return result.json()
        else:
            raise Exception(
                f"Could not update task {task_id}. Got {result}: {result.text}"
            )



import os
import subprocess
from pathlib import Path

from pythonanywhere.snakesay import snakesay


class Virtualenv:
    def __init__(self, domain, python_version):
        self.domain = domain
        self.python_version = python_version
        self.path = Path(os.environ["WORKON_HOME"]) / domain

    def __eq__(self, other):
        return self.domain == other.domain and self.python_version == other.python_version

    def create(self, nuke):
        print(snakesay(f"Creating virtualenv with Python{self.python_version}"))
        command = f"mkvirtualenv --python=python{self.python_version} {self.domain}"
        if nuke:
            command = f"rmvirtualenv {self.domain} && {command}"
        subprocess.check_call(["bash", "-c", f"source virtualenvwrapper.sh && {command}"])
        return self

    def pip_install(self, packages):
        print(snakesay(f"Pip installing {packages} (this may take a couple of minutes)"))
        commands = [str(self.path / "bin/pip"), "install"] + packages.split()
        subprocess.check_call(commands)

    def get_version(self, package_name):
        commands = [str(self.path / "bin/pip"), "show", package_name]
        output = subprocess.check_output(commands).decode()
        for line in output.splitlines():
            if line.startswith("Version: "):
                return line.split()[1]



#!C:\Users\dev-admin\perseverance-python-buildout\croot\pythonanywhere_1699565098275\_h_env\python.exe
"""Get current scheduled task's specs file by task id.

Available specs are: command, enabled, interval, hour, minute, printable-time,
logfile, expiry. If no option specified, script will output all mentioned specs.

Note that logfile query provides path for current (last) logfile. There may be
several logfiles for each task.
If task has been updated (e.g. by `pa_update_scheduled_task.py` script) logfile
name has been changed too, but the file will not be created until first execution
of the task. Thus getting logfile path via API call does not necessarily mean the
file exists on the server yet.

Usage:
  pa_get_scheduled_task_specs.py <id> [--command] [--enabled] [--interval]
                                      [--hour] [--minute] [--printable-time]
                                      [--logfile] [--expiry]
                                      [--snakesay | --no-spec]

Options:
  -h, --help                  Prints this message
  -c, --command               Prints task's command
  -e, --enabled               Prints task's enabled status (True or False)
  -i, --interval              Prints task's frequency (daily or hourly)
  -l, --logfile               Prints task's current log file path
  -m, --minute                Prints task's scheduled minute
  -o, --hour                  Prints task's scheduled hour (if daily)
  -p, --printable-time        Prints task's scheduled time
  -x, --expiry                Prints task's expiry date
  -n, --no-spec               Prints only values without spec names
  -s, --snakesay              Turns on snakesay... because why not

Note:
  Task <id> may be found using pa_get_scheduled_tasks_list.py script.

Example:
  Get all specs for task with id 42:

    pa_get_scheduled_task_specs 42

  Get only logfile name for task with id 42:

    pa_get_scheduled_task_specs 42 --logfile --no-spec"""

from docopt import docopt
from tabulate import tabulate

from pythonanywhere.scripts_commons import ScriptSchema, get_logger, get_task_from_id
from pythonanywhere.snakesay import snakesay


def main(*, task_id, **kwargs):
    logger = get_logger(set_info=True)
    task = get_task_from_id(task_id)

    print_snake = kwargs.pop("snake")
    print_only_values = kwargs.pop("no_spec")

    specs = (
        {spec: getattr(task, spec) for spec in kwargs if kwargs[spec]}
        if any([val for val in kwargs.values()])
        else {spec: getattr(task, spec) for spec in kwargs}
    )

    # get user path instead of server path:
    if specs.get("logfile"):
        specs.update({"logfile": task.logfile.replace(f"/user/{task.user}/files", "")})

    intro = f"Task {task_id} specs: "
    if print_only_values:
        specs = "\n".join([str(val) for val in specs.values()])
        logger.info(specs)
    elif print_snake:
        specs = [f"<{spec}>: {value}" for spec, value in specs.items()]
        specs.sort()
        logger.info(snakesay(intro + ", ".join(specs)))
    else:
        table = [[spec, val] for spec, val in specs.items()]
        table.sort(key=lambda x: x[0])
        logger.info(intro)
        logger.info(tabulate(table, tablefmt="simple"))


if __name__ == "__main__":
    schema = ScriptSchema(
        {
            "<id>": ScriptSchema.id_required,
            "--command": ScriptSchema.boolean,
            "--enabled": ScriptSchema.boolean,
            "--expiry": ScriptSchema.boolean,
            "--hour": ScriptSchema.boolean,
            "--interval": ScriptSchema.boolean,
            "--logfile": ScriptSchema.boolean,
            "--minute": ScriptSchema.boolean,
            "--printable-time": ScriptSchema.boolean,
            "--no-spec": ScriptSchema.boolean,
            "--snakesay": ScriptSchema.boolean,
        }
    )
    arguments = schema.validate_user_input(
        docopt(__doc__),
        conversions={
            "id": "task_id",
            "no-": "no_",
            "printable-": "printable_",
            "snakesay": "snake",
        },
    )

    main(**arguments)



"""Interface speaking with PythonAnywhere API providing methods for
files.  *Don't use* `Files` :class: in helper scripts, use
`pythonanywhere.files.Path` class instead."""

import getpass
from urllib.parse import urljoin

from pythonanywhere.api.base import call_api, get_api_endpoint


class Files:
    """ Interface for PythonAnywhere files API.

    Uses `pythonanywhere.api.base` :method: `get_api_endpoint` to
    create url, which is stored in a class variable `Files.base_url`,
    then calls `call_api` with appropriate arguments to execute files
    action.

    Covers:
    - GET, POST and DELETE for files path endpoint
    - POST, GET and DELETE for files sharing endpoint
    - GET for tree endpoint

    "path" methods:
    - use :method: `Files.path_get` to get contents of file or
    directory from `path`
    - use :method: `Files.path_post` to upload or update file at given
    `dest_path` using contents from `source`
    - use :method: `Files.path_delete` to delete file/directory on on
    given `path`

    "sharing" methods:
    - use :method: `Files.sharing_post` to enable sharing a file from
    `path` (if not shared before) and get a link to it
    - use :method: `Files.sharing_get` to get sharing url for `path`
    - use :method: `Files.sharing_delete` to disable sharing for
    `path`

    "tree" method:
    - use :method: `Files.tree_get` to get list of regular files and
    subdirectories of a directory at `path` (limited to 1000 results)
    """

    base_url = get_api_endpoint().format(username=getpass.getuser(), flavor="files")
    path_endpoint = urljoin(base_url, "path")
    sharing_endpoint = urljoin(base_url, "sharing/")
    tree_endpoint = urljoin(base_url, "tree/")

    def _error_msg(self, result):
        """TODO: error responses should be unified at the API side """

        if "application/json" in result.headers.get("content-type", ""):
            jsn = result.json()
            msg = jsn.get("detail") or jsn.get("message") or jsn.get("error", "")
            return f": {msg}"
        return ""

    def path_get(self, path):
        """Returns dictionary of directory contents when `path` is an
        absolute path to of an existing directory or file contents if
        `path` is an absolute path to an existing file -- both
        available to the PythonAnywhere user.  Raises when `path` is
        invalid or unavailable."""

        url = f"{self.path_endpoint}{path}"

        result = call_api(url, "GET")

        if result.status_code == 200:
            if "application/json" in result.headers.get("content-type", ""):
                return result.json()
            return result.content

        raise Exception(
            f"GET to fetch contents of {url} failed, got {result}{self._error_msg(result)}"
        )

    def path_post(self, dest_path, content):
        """Uploads contents of `content` to `dest_path` which should be
        a valid absolute path of a file available to a PythonAnywhere
        user.  If `dest_path` contains directories which don't exist
        yet, they will be created.

        Returns 200 if existing file on PythonAnywhere has been
        updated with `source` contents, or 201 if file from
        `dest_path` has been created with those contents."""

        url = f"{self.path_endpoint}{dest_path}"

        result = call_api(url, "POST", files={"content": content})

        if result.ok:
            return result.status_code

        raise Exception(
            f"POST to upload contents to {url} failed, got {result}{self._error_msg(result)}"
        )

    def path_delete(self, path):
        """Deletes the file at specified `path` (if file is a
        directory it will be deleted as well).

        Returns 204 on sucess, raises otherwise."""

        url = f"{self.path_endpoint}{path}"

        result = call_api(url, "DELETE")

        if result.status_code == 204:
            return result.status_code

        raise Exception(
            f"DELETE on {url} failed, got {result}{self._error_msg(result)}"
        )

    def sharing_post(self, path):
        """Starts sharing a file at `path`.

        Returns a tuple with a status code and sharing link on
        success, raises otherwise.  Status code is 201 on success, 200
        if file has been already shared."""

        url = self.sharing_endpoint

        result = call_api(url, "POST", json={"path": path})

        if result.ok:
            return result.status_code, result.json()["url"]

        raise Exception(
            f"POST to {url} to share '{path}' failed, got {result}{self._error_msg(result)}"
        )

    def sharing_get(self, path):
        """Checks sharing status for a `path`.

        Returns url with sharing link if file is shared or an empty
        string otherwise."""

        url = f"{self.sharing_endpoint}?path={path}"

        result = call_api(url, "GET")

        return result.json()["url"] if result.ok else ""

    def sharing_delete(self, path):
        """Stops sharing file at `path`.

        Returns 204 on successful unshare."""

        url = f"{self.sharing_endpoint}?path={path}"

        result = call_api(url, "DELETE")

        return result.status_code

    def tree_get(self, path):
        """Returns list of absolute paths of regular files and
        subdirectories of a directory at `path`.  Result is limited to
        1000 items.

        Raises if `path` does not point to an existing directory."""

        url = f"{self.tree_endpoint}?path={path}"

        result = call_api(url, "GET")

        if result.ok:
            return result.json()

        raise Exception(f"GET to {url} failed, got {result}{self._error_msg(result)}")
